@@ -558,7 +558,7 @@ func TestLowerPrivateGetterSetterNext(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					get #foo() { return this.foo }
 					set #bar(val) { this.bar = val }
 					get #prop() { return this.prop }
@@ -692,7 +692,7 @@ func TestLowerPrivateMethodNext(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					#field
 					#method() {}
 					baseline() {
@@ -959,6 +959,30 @@ func TestLowerAsyncSuperES2017NoBundle(t *testing.T) {
 						]
 					}
 				}
+
+				// This covers a bug that caused a compiler crash
+				let fn = async () => class extends Base {
+					a = super.a
+					b = () => super.b
+					c() { return super.c }
+					d() { return () => super.d }
+				}
+
+				// This covers a bug that generated bad code
+				class Derived2 extends Base {
+					async a() { return class { [super.foo] = 123 } }
+					b = async () => class { [super.foo] = 123 }
+				}
+
+				// This covers putting the generated temporary variable inside the loop
+				for (let i = 0; i < 3; i++) {
+					objs.push({
+						__proto__: {
+							foo() { return i },
+						},
+						async bar() { return super.foo() },
+					})
+				}
 			`,
 		},
 		entryPaths: []string{"/entry.js"},
@@ -1008,6 +1032,30 @@ func TestLowerAsyncSuperES2016NoBundle(t *testing.T) {
 						]
 					}
 				}
+
+				// This covers a bug that caused a compiler crash
+				let fn = async () => class extends Base {
+					a = super.a
+					b = () => super.b
+					c() { return super.c }
+					d() { return () => super.d }
+				}
+
+				// This covers a bug that generated bad code
+				class Derived2 extends Base {
+					async a() { return class { [super.foo] = 123 } }
+					b = async () => class { [super.foo] = 123 }
+				}
+
+				// This covers putting the generated temporary variable inside the loop
+				for (let i = 0; i < 3; i++) {
+					objs.push({
+						__proto__: {
+							foo() { return i },
+						},
+						async bar() { return super.foo() },
+					})
+				}
 			`,
 		},
 		entryPaths: []string{"/entry.js"},
@@ -1057,6 +1105,20 @@ func TestLowerStaticAsyncSuperES2021NoBundle(t *testing.T) {
 						]
 					}
 				}
+
+				// This covers a bug that caused a compiler crash
+				let fn = async () => class extends Base {
+					static a = super.a
+					static b = () => super.b
+					static c() { return super.c }
+					static d() { return () => super.d }
+				}
+
+				// This covers a bug that generated bad code
+				class Derived2 extends Base {
+					static async a() { return class { [super.foo] = 123 } }
+					static b = async () => class { [super.foo] = 123 }
+				}
 			`,
 		},
 		entryPaths: []string{"/entry.js"},
@@ -1106,6 +1168,20 @@ func TestLowerStaticAsyncSuperES2016NoBundle(t *testing.T) {
 						]
 					}
 				}
+
+				// This covers a bug that caused a compiler crash
+				let fn = async () => class extends Base {
+					static a = super.a
+					static b = () => super.b
+					static c() { return super.c }
+					static d() { return () => super.d }
+				}
+
+				// This covers a bug that generated bad code
+				class Derived2 extends Base {
+					static async a() { return class { [super.foo] = 123 } }
+					static b = async () => class { [super.foo] = 123 }
+				}
 			`,
 		},
 		entryPaths: []string{"/entry.js"},
@@ -1394,6 +1470,90 @@ func TestLowerStaticAsyncArrowSuperSetterES2016(t *testing.T) {
 	})
 }
 
+func TestLowerPrivateSuperES2022(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				export { default as foo1 } from "./foo1"
+				export { default as foo2 } from "./foo2"
+				export { default as foo3 } from "./foo3"
+				export { default as foo4 } from "./foo4"
+				export { default as foo5 } from "./foo5"
+				export { default as foo6 } from "./foo6"
+				export { default as foo7 } from "./foo7"
+				export { default as foo8 } from "./foo8"
+			`,
+			"/foo1.js": `export default class extends x { #foo() { super.foo() } }`,
+			"/foo2.js": `export default class extends x { #foo() { super.foo++ } }`,
+			"/foo3.js": `export default class extends x { static #foo() { super.foo() } }`,
+			"/foo4.js": `export default class extends x { static #foo() { super.foo++ } }`,
+			"/foo5.js": `export default class extends x { #foo = () => { super.foo() } }`,
+			"/foo6.js": `export default class extends x { #foo = () => { super.foo++ } }`,
+			"/foo7.js": `export default class extends x { static #foo = () => { super.foo() } }`,
+			"/foo8.js": `export default class extends x { static #foo = () => { super.foo++ } }`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			UnsupportedJSFeatures: es(2022),
+			AbsOutputFile:         "/out.js",
+		},
+	})
+}
+
+func TestLowerPrivateSuperES2021(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				export { default as foo1 } from "./foo1"
+				export { default as foo2 } from "./foo2"
+				export { default as foo3 } from "./foo3"
+				export { default as foo4 } from "./foo4"
+				export { default as foo5 } from "./foo5"
+				export { default as foo6 } from "./foo6"
+				export { default as foo7 } from "./foo7"
+				export { default as foo8 } from "./foo8"
+			`,
+			"/foo1.js": `export default class extends x { #foo() { super.foo() } }`,
+			"/foo2.js": `export default class extends x { #foo() { super.foo++ } }`,
+			"/foo3.js": `export default class extends x { static #foo() { super.foo() } }`,
+			"/foo4.js": `export default class extends x { static #foo() { super.foo++ } }`,
+			"/foo5.js": `export default class extends x { #foo = () => { super.foo() } }`,
+			"/foo6.js": `export default class extends x { #foo = () => { super.foo++ } }`,
+			"/foo7.js": `export default class extends x { static #foo = () => { super.foo() } }`,
+			"/foo8.js": `export default class extends x { static #foo = () => { super.foo++ } }`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			UnsupportedJSFeatures: es(2021),
+			AbsOutputFile:         "/out.js",
+		},
+	})
+}
+
+// https://github.com/evanw/esbuild/issues/2158
+func TestLowerPrivateSuperStaticBundleIssue2158(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				export class Foo extends Object {
+					static FOO;
+					constructor() {
+						super();
+					}
+					#foo;
+				}
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
 func TestLowerClassField2020NoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -2041,7 +2201,7 @@ func TestLowerNullishCoalescingAssignmentIssue1493(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class A {
+				export class A {
 					#a;
 					f() {
 						this.#a ??= 1;
