@@ -1417,3 +1417,100 @@ func TestTsconfigOverriddenTargetWarning(t *testing.T) {
 `,
 	})
 }
+
+func TestTsConfigNoBaseURLExtendsPaths(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.ts": `
+				import { foo } from "foo"
+				console.log(foo)
+			`,
+			"/Users/user/project/lib/foo.ts": `
+				export let foo = 123
+			`,
+			"/Users/user/project/tsconfig.json": `{
+				"extends": "./base/defaults"
+			}`,
+			"/Users/user/project/base/defaults.json": `{
+				"compilerOptions": {
+					"paths": {
+						"*": ["lib/*"]
+					}
+				}
+			}`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+		expectedScanLog: `Users/user/project/base/defaults.json: WARNING: Non-relative path "lib/*" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+Users/user/project/src/entry.ts: ERROR: Could not resolve "foo"
+NOTE: You can mark the path "foo" as external to exclude it from the bundle, which will remove this error.
+`,
+	})
+}
+
+func TestTsConfigBaseURLExtendsPaths(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.ts": `
+				import { foo } from "foo"
+				console.log(foo)
+			`,
+			"/Users/user/project/lib/foo.ts": `
+				export let foo = 123
+			`,
+			"/Users/user/project/tsconfig.json": `{
+				"extends": "./base/defaults",
+				"compilerOptions": {
+					"baseUrl": "."
+				}
+			}`,
+			"/Users/user/project/base/defaults.json": `{
+				"compilerOptions": {
+					"paths": {
+						"*": ["lib/*"]
+					}
+				}
+			}`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
+
+func TestTsConfigPathsExtendsBaseURL(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.ts": `
+				import { foo } from "foo"
+				console.log(foo)
+			`,
+			"/Users/user/project/base/test/lib/foo.ts": `
+				export let foo = 123
+			`,
+			"/Users/user/project/tsconfig.json": `{
+				"extends": "./base/defaults",
+				"compilerOptions": {
+					"paths": {
+						"*": ["lib/*"]
+					}
+				}
+			}`,
+			"/Users/user/project/base/defaults.json": `{
+				"compilerOptions": {
+					"baseUrl": "test"
+				}
+			}`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
