@@ -33,7 +33,7 @@ type TSConfigJSON struct {
 	// the wildcard is substituted into the fallback path. The keys represent
 	// module-style path names and the fallback paths are relative to the
 	// "baseUrl" value in the "tsconfig.json" file.
-	Paths map[string][]string
+	Paths *TSConfigPaths
 
 	TSTarget                       *config.TSTarget
 	JSXFactory                     []string
@@ -43,6 +43,19 @@ type TSConfigJSON struct {
 	PreserveValueImports           bool
 }
 
+type TSConfigPath struct {
+	Text string
+	Loc  logger.Loc
+}
+
+type TSConfigPaths struct {
+	Map map[string][]TSConfigPath
+
+	// This may be different from the original "tsconfig.json" source if the
+	// "paths" value is from another file via an "extends" clause.
+	Source logger.Source
+}
+
 func ParseTSConfigJSON(
 	log logger.Log,
 	source logger.Source,
@@ -122,6 +135,7 @@ func ParseTSConfigJSON(
 				ok := true
 
 				// See https://www.typescriptlang.org/tsconfig#target
+				targetIsAtLeastES2022 := false
 				switch strings.ToLower(value) {
 				case "es5":
 					constraints[compat.ES] = []int{5}
@@ -141,8 +155,9 @@ func ParseTSConfigJSON(
 					constraints[compat.ES] = []int{2021}
 				case "es2022":
 					constraints[compat.ES] = []int{2022}
+					targetIsAtLeastES2022 = true
 				case "esnext":
-					// Nothing to do in this case
+					targetIsAtLeastES2022 = true
 				default:
 					ok = false
 					if !helpers.IsInsideNodeModules(source.KeyPath.Text) {
@@ -158,6 +173,7 @@ func ParseTSConfigJSON(
 						Range:                 r,
 						Target:                value,
 						UnsupportedJSFeatures: compat.UnsupportedJSFeatures(constraints),
+						TargetIsAtLeastES2022: targetIsAtLeastES2022,
 					}
 				}
 			}
@@ -193,7 +209,7 @@ func ParseTSConfigJSON(
 				} else {
 					result.BaseURLForPaths = "."
 				}
-				result.Paths = make(map[string][]string)
+				result.Paths = &TSConfigPaths{Source: source, Map: make(map[string][]TSConfigPath)}
 				for _, prop := range paths.Properties {
 					if key, ok := getString(prop.Key); ok {
 						if !isValidTSConfigPathPattern(key, log, &source, &tracker, prop.Key.Loc) {
@@ -224,9 +240,8 @@ func ParseTSConfigJSON(
 						if array, ok := prop.ValueOrNil.Data.(*js_ast.EArray); ok {
 							for _, item := range array.Items {
 								if str, ok := getString(item); ok {
-									if isValidTSConfigPathPattern(str, log, &source, &tracker, item.Loc) &&
-										(hasBaseURL || isValidTSConfigPathNoBaseURLPattern(str, log, &source, &tracker, item.Loc)) {
-										result.Paths[key] = append(result.Paths[key], str)
+									if isValidTSConfigPathPattern(str, log, &source, &tracker, item.Loc) {
+										result.Paths.Map[key] = append(result.Paths.Map[key], TSConfigPath{Text: str, Loc: item.Loc})
 									}
 								}
 							}
@@ -278,7 +293,7 @@ func isSlash(c byte) bool {
 	return c == '/' || c == '\\'
 }
 
-func isValidTSConfigPathNoBaseURLPattern(text string, log logger.Log, source *logger.Source, tracker *logger.LineColumnTracker, loc logger.Loc) bool {
+func isValidTSConfigPathNoBaseURLPattern(text string, log logger.Log, source *logger.Source, tracker **logger.LineColumnTracker, loc logger.Loc) bool {
 	var c0 byte
 	var c1 byte
 	var c2 byte
@@ -315,7 +330,11 @@ func isValidTSConfigPathNoBaseURLPattern(text string, log logger.Log, source *lo
 	}
 
 	r := source.RangeOfString(loc)
-	log.Add(logger.Warning, tracker, r, fmt.Sprintf(
+	if *tracker == nil {
+		t := logger.MakeLineColumnTracker(source)
+		*tracker = &t
+	}
+	log.Add(logger.Warning, *tracker, r, fmt.Sprintf(
 		"Non-relative path %q is not allowed when \"baseUrl\" is not set (did you forget a leading \"./\"?)", text))
 	return false
 }
