@@ -34,68 +34,169 @@ func newTransformOptions() api.TransformOptions {
 type parseOptionsKind uint8
 
 const (
+	// This means we're parsing it for our own internal use
 	kindInternal parseOptionsKind = iota
+
+	// This means the result is returned through a public API
 	kindExternal
 )
 
+type parseOptionsExtras struct {
+	metafile    *string
+	mangleCache *string
+}
+
+func isBoolFlag(arg string, flag string) bool {
+	if strings.HasPrefix(arg, flag) {
+		remainder := arg[len(flag):]
+		return len(remainder) == 0 || remainder[0] == '='
+	}
+	return false
+}
+
+func parseBoolFlag(arg string, defaultValue bool) (bool, *cli_helpers.ErrorWithNote) {
+	equals := strings.IndexByte(arg, '=')
+	if equals == -1 {
+		return defaultValue, nil
+	}
+	value := arg[equals+1:]
+	switch value {
+	case "false":
+		return false, nil
+	case "true":
+		return true, nil
+	}
+	return false, cli_helpers.MakeErrorWithNote(
+		fmt.Sprintf("Invalid value %q in %q", value, arg),
+		"Valid values are \"true\" or \"false\".",
+	)
+}
+
 func parseOptionsImpl(
 	osArgs []string,
 	buildOpts *api.BuildOptions,
 	transformOpts *api.TransformOptions,
 	kind parseOptionsKind,
-) (err *cli_helpers.ErrorWithNote, metafile *string) {
+) (extras parseOptionsExtras, err *cli_helpers.ErrorWithNote) {
 	hasBareSourceMapFlag := false
 
 	// Parse the arguments now that we know what we're parsing
 	for _, arg := range osArgs {
 		switch {
-		case arg == "--bundle" && buildOpts != nil:
-			buildOpts.Bundle = true
-
-		case arg == "--preserve-symlinks" && buildOpts != nil:
-			buildOpts.PreserveSymlinks = true
-
-		case arg == "--splitting" && buildOpts != nil:
-			buildOpts.Splitting = true
-
-		case arg == "--allow-overwrite" && buildOpts != nil:
-			buildOpts.AllowOverwrite = true
-
-		case arg == "--watch" && buildOpts != nil:
-			buildOpts.Watch = &api.WatchMode{}
-
-		case arg == "--minify":
-			if buildOpts != nil {
-				buildOpts.MinifySyntax = true
-				buildOpts.MinifyWhitespace = true
-				buildOpts.MinifyIdentifiers = true
+		case isBoolFlag(arg, "--bundle") && buildOpts != nil:
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
 			} else {
-				transformOpts.MinifySyntax = true
-				transformOpts.MinifyWhitespace = true
-				transformOpts.MinifyIdentifiers = true
+				buildOpts.Bundle = value
 			}
 
-		case arg == "--minify-syntax":
-			if buildOpts != nil {
-				buildOpts.MinifySyntax = true
+		case isBoolFlag(arg, "--preserve-symlinks") && buildOpts != nil:
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
 			} else {
-				transformOpts.MinifySyntax = true
+				buildOpts.PreserveSymlinks = value
 			}
 
-		case arg == "--minify-whitespace":
-			if buildOpts != nil {
-				buildOpts.MinifyWhitespace = true
+		case isBoolFlag(arg, "--splitting") && buildOpts != nil:
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
 			} else {
-				transformOpts.MinifyWhitespace = true
+				buildOpts.Splitting = value
 			}
 
-		case arg == "--minify-identifiers":
-			if buildOpts != nil {
-				buildOpts.MinifyIdentifiers = true
+		case isBoolFlag(arg, "--allow-overwrite") && buildOpts != nil:
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
 			} else {
-				transformOpts.MinifyIdentifiers = true
+				buildOpts.AllowOverwrite = value
 			}
 
+		case isBoolFlag(arg, "--watch") && buildOpts != nil:
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if value {
+				buildOpts.Watch = &api.WatchMode{}
+			} else {
+				buildOpts.Watch = nil
+			}
+
+		case isBoolFlag(arg, "--minify"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.MinifySyntax = value
+				buildOpts.MinifyWhitespace = value
+				buildOpts.MinifyIdentifiers = value
+			} else {
+				transformOpts.MinifySyntax = value
+				transformOpts.MinifyWhitespace = value
+				transformOpts.MinifyIdentifiers = value
+			}
+
+		case isBoolFlag(arg, "--minify-syntax"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.MinifySyntax = value
+			} else {
+				transformOpts.MinifySyntax = value
+			}
+
+		case isBoolFlag(arg, "--minify-whitespace"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.MinifyWhitespace = value
+			} else {
+				transformOpts.MinifyWhitespace = value
+			}
+
+		case isBoolFlag(arg, "--minify-identifiers"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.MinifyIdentifiers = value
+			} else {
+				transformOpts.MinifyIdentifiers = value
+			}
+
+		case isBoolFlag(arg, "--mangle-quoted"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else {
+				var mangleQuoted *api.MangleQuoted
+				if buildOpts != nil {
+					mangleQuoted = &buildOpts.MangleQuoted
+				} else {
+					mangleQuoted = &transformOpts.MangleQuoted
+				}
+				if value {
+					*mangleQuoted = api.MangleQuotedTrue
+				} else {
+					*mangleQuoted = api.MangleQuotedFalse
+				}
+			}
+
+		case strings.HasPrefix(arg, "--mangle-props="):
+			value := arg[len("--mangle-props="):]
+			if buildOpts != nil {
+				buildOpts.MangleProps = value
+			} else {
+				transformOpts.MangleProps = value
+			}
+
+		case strings.HasPrefix(arg, "--reserve-props="):
+			value := arg[len("--reserve-props="):]
+			if buildOpts != nil {
+				buildOpts.ReserveProps = value
+			} else {
+				transformOpts.ReserveProps = value
+			}
+
+		case strings.HasPrefix(arg, "--mangle-cache=") && buildOpts != nil && kind == kindInternal:
+			value := arg[len("--mangle-cache="):]
+			extras.mangleCache = &value
+
 		case strings.HasPrefix(arg, "--drop:"):
 			value := arg[len("--drop:"):]
 			switch value {
@@ -112,10 +213,10 @@ func parseOptionsImpl(
 					transformOpts.Drop |= api.DropDebugger
 				}
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"console\" or \"debugger\".",
-				), nil
+				)
 			}
 
 		case strings.HasPrefix(arg, "--legal-comments="):
@@ -133,10 +234,10 @@ func parseOptionsImpl(
 			case "external":
 				legalComments = api.LegalCommentsExternal
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"none\", \"inline\", \"eof\", \"linked\", or \"external\".",
-				), nil
+				)
 			}
 			if buildOpts != nil {
 				buildOpts.LegalComments = legalComments
@@ -158,44 +259,45 @@ func parseOptionsImpl(
 			case "utf8":
 				*value = api.CharsetUTF8
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", name, arg),
 					"Valid values are \"ascii\" or \"utf8\".",
-				), nil
+				)
 			}
 
-		case strings.HasPrefix(arg, "--tree-shaking="):
-			var value *api.TreeShaking
-			if buildOpts != nil {
-				value = &buildOpts.TreeShaking
+		case isBoolFlag(arg, "--tree-shaking"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
 			} else {
-				value = &transformOpts.TreeShaking
-			}
-			name := arg[len("--tree-shaking="):]
-			switch name {
-			case "false":
-				*value = api.TreeShakingFalse
-			case "true":
-				*value = api.TreeShakingTrue
-			default:
-				return cli_helpers.MakeErrorWithNote(
-					fmt.Sprintf("Invalid value %q in %q", name, arg),
-					"Valid values are \"true\" or \"false\".",
-				), nil
+				var treeShaking *api.TreeShaking
+				if buildOpts != nil {
+					treeShaking = &buildOpts.TreeShaking
+				} else {
+					treeShaking = &transformOpts.TreeShaking
+				}
+				if value {
+					*treeShaking = api.TreeShakingTrue
+				} else {
+					*treeShaking = api.TreeShakingFalse
+				}
 			}
 
-		case arg == "--ignore-annotations":
-			if buildOpts != nil {
-				buildOpts.IgnoreAnnotations = true
+		case isBoolFlag(arg, "--ignore-annotations"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.IgnoreAnnotations = value
 			} else {
-				transformOpts.IgnoreAnnotations = true
+				transformOpts.IgnoreAnnotations = value
 			}
 
-		case arg == "--keep-names":
-			if buildOpts != nil {
-				buildOpts.KeepNames = true
+		case isBoolFlag(arg, "--keep-names"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.KeepNames = value
 			} else {
-				transformOpts.KeepNames = true
+				transformOpts.KeepNames = value
 			}
 
 		case arg == "--sourcemap":
@@ -210,6 +312,8 @@ func parseOptionsImpl(
 			value := arg[len("--sourcemap="):]
 			var sourcemap api.SourceMap
 			switch value {
+			case "linked":
+				sourcemap = api.SourceMapLinked
 			case "inline":
 				sourcemap = api.SourceMapInline
 			case "external":
@@ -217,10 +321,10 @@ func parseOptionsImpl(
 			case "both":
 				sourcemap = api.SourceMapInlineAndExternal
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"inline\", \"external\", or \"both\".",
-				), nil
+				)
 			}
 			if buildOpts != nil {
 				buildOpts.Sourcemap = sourcemap
@@ -237,24 +341,21 @@ func parseOptionsImpl(
 				transformOpts.SourceRoot = sourceRoot
 			}
 
-		case strings.HasPrefix(arg, "--sources-content="):
-			value := arg[len("--sources-content="):]
-			var sourcesContent api.SourcesContent
-			switch value {
-			case "false":
-				sourcesContent = api.SourcesContentExclude
-			case "true":
-				sourcesContent = api.SourcesContentInclude
-			default:
-				return cli_helpers.MakeErrorWithNote(
-					fmt.Sprintf("Invalid value %q in %q", value, arg),
-					"Valid values are \"true\" or \"false\".",
-				), nil
-			}
-			if buildOpts != nil {
-				buildOpts.SourcesContent = sourcesContent
+		case isBoolFlag(arg, "--sources-content"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
 			} else {
-				transformOpts.SourcesContent = sourcesContent
+				var sourcesContent *api.SourcesContent
+				if buildOpts != nil {
+					sourcesContent = &buildOpts.SourcesContent
+				} else {
+					sourcesContent = &transformOpts.SourcesContent
+				}
+				if value {
+					*sourcesContent = api.SourcesContentInclude
+				} else {
+					*sourcesContent = api.SourcesContentExclude
+				}
 			}
 
 		case strings.HasPrefix(arg, "--sourcefile="):
@@ -290,9 +391,9 @@ func parseOptionsImpl(
 			buildOpts.Metafile = true
 
 		case strings.HasPrefix(arg, "--metafile=") && buildOpts != nil && kind == kindInternal:
-			metafilePath := arg[len("--metafile="):]
+			value := arg[len("--metafile="):]
 			buildOpts.Metafile = true
-			metafile = &metafilePath
+			extras.metafile = &value
 
 		case strings.HasPrefix(arg, "--outfile=") && buildOpts != nil:
 			buildOpts.Outfile = arg[len("--outfile="):]
@@ -322,11 +423,11 @@ func parseOptionsImpl(
 			value := arg[len("--define:"):]
 			equals := strings.IndexByte(value, '=')
 			if equals == -1 {
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Missing \"=\" in %q", arg),
 					"You need to use \"=\" to specify both the original value and the replacement value. "+
 						"For example, \"--define:DEBUG=true\" replaces \"DEBUG\" with \"true\".",
-				), nil
+				)
 			}
 			if buildOpts != nil {
 				buildOpts.Define[value[:equals]] = value[equals+1:]
@@ -346,16 +447,16 @@ func parseOptionsImpl(
 			value := arg[len("--loader:"):]
 			equals := strings.IndexByte(value, '=')
 			if equals == -1 {
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Missing \"=\" in %q", arg),
 					"You need to specify the file extension that the loader applies to. "+
 						"For example, \"--loader:.js=jsx\" applies the \"jsx\" loader to files with the \".js\" extension.",
-				), nil
+				)
 			}
 			ext, text := value[:equals], value[equals+1:]
 			loader, err := cli_helpers.ParseLoader(text)
 			if err != nil {
-				return err, nil
+				return parseOptionsExtras{}, err
 			}
 			buildOpts.Loader[ext] = loader
 
@@ -363,14 +464,14 @@ func parseOptionsImpl(
 			value := arg[len("--loader="):]
 			loader, err := cli_helpers.ParseLoader(value)
 			if err != nil {
-				return err, nil
+				return parseOptionsExtras{}, err
 			}
 			if loader == api.LoaderFile {
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("%q is not supported when transforming stdin", arg),
 					"Using esbuild to transform stdin only generates one output file, so you cannot use the \"file\" loader "+
 						"since that needs to generate two output files.",
-				), nil
+				)
 			}
 			if buildOpts != nil {
 				if buildOpts.Stdin == nil {
@@ -384,7 +485,7 @@ func parseOptionsImpl(
 		case strings.HasPrefix(arg, "--target="):
 			target, engines, err := parseTargets(splitWithEmptyCheck(arg[len("--target="):], ","), arg)
 			if err != nil {
-				return err, nil
+				return parseOptionsExtras{}, err
 			}
 			if buildOpts != nil {
 				buildOpts.Target = target
@@ -398,11 +499,11 @@ func parseOptionsImpl(
 			value := arg[len("--out-extension:"):]
 			equals := strings.IndexByte(value, '=')
 			if equals == -1 {
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Missing \"=\" in %q", arg),
 					"You need to use either \"--out-extension:.js=...\" or \"--out-extension:.css=...\" "+
 						"to specify the file type that the output extension applies to .",
-				), nil
+				)
 			}
 			if buildOpts.OutExtensions == nil {
 				buildOpts.OutExtensions = make(map[string]string)
@@ -419,10 +520,10 @@ func parseOptionsImpl(
 			case "neutral":
 				buildOpts.Platform = api.PlatformNeutral
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"browser\", \"node\", or \"neutral\".",
-				), nil
+				)
 			}
 
 		case strings.HasPrefix(arg, "--format="):
@@ -447,10 +548,10 @@ func parseOptionsImpl(
 					transformOpts.Format = api.FormatESModule
 				}
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"iife\", \"cjs\", or \"esm\".",
-				), nil
+				)
 			}
 
 		case strings.HasPrefix(arg, "--external:") && buildOpts != nil:
@@ -468,10 +569,10 @@ func parseOptionsImpl(
 			case "preserve":
 				mode = api.JSXModePreserve
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"transform\" or \"preserve\".",
-				), nil
+				)
 			}
 			if buildOpts != nil {
 				buildOpts.JSXMode = mode
@@ -505,10 +606,10 @@ func parseOptionsImpl(
 			value := arg[len("--banner:"):]
 			equals := strings.IndexByte(value, '=')
 			if equals == -1 {
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Missing \"=\" in %q", arg),
 					"You need to use either \"--banner:js=...\" or \"--banner:css=...\" to specify the language that the banner applies to.",
-				), nil
+				)
 			}
 			buildOpts.Banner[value[:equals]] = value[equals+1:]
 
@@ -516,10 +617,10 @@ func parseOptionsImpl(
 			value := arg[len("--footer:"):]
 			equals := strings.IndexByte(value, '=')
 			if equals == -1 {
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Missing \"=\" in %q", arg),
 					"You need to use either \"--footer:js=...\" or \"--footer:css=...\" to specify the language that the footer applies to.",
-				), nil
+				)
 			}
 			buildOpts.Footer[value[:equals]] = value[equals+1:]
 
@@ -527,10 +628,10 @@ func parseOptionsImpl(
 			value := arg[len("--log-limit="):]
 			limit, err := strconv.Atoi(value)
 			if err != nil || limit < 0 {
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"The log limit must be a non-negative integer.",
-				), nil
+				)
 			}
 			if buildOpts != nil {
 				buildOpts.LogLimit = limit
@@ -539,24 +640,21 @@ func parseOptionsImpl(
 			}
 
 			// Make sure this stays in sync with "PrintErrorToStderr"
-		case strings.HasPrefix(arg, "--color="):
-			value := arg[len("--color="):]
-			var color api.StderrColor
-			switch value {
-			case "false":
-				color = api.ColorNever
-			case "true":
-				color = api.ColorAlways
-			default:
-				return cli_helpers.MakeErrorWithNote(
-					fmt.Sprintf("Invalid value %q in %q", value, arg),
-					"Valid values are \"true\" or \"false\".",
-				), nil
-			}
-			if buildOpts != nil {
-				buildOpts.Color = color
+		case isBoolFlag(arg, "--color"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
 			} else {
-				transformOpts.Color = color
+				var color *api.StderrColor
+				if buildOpts != nil {
+					color = &buildOpts.Color
+				} else {
+					color = &transformOpts.Color
+				}
+				if value {
+					*color = api.ColorAlways
+				} else {
+					*color = api.ColorNever
+				}
 			}
 
 		// Make sure this stays in sync with "PrintErrorToStderr"
@@ -577,10 +675,10 @@ func parseOptionsImpl(
 			case "silent":
 				logLevel = api.LogLevelSilent
 			default:
-				return cli_helpers.MakeErrorWithNote(
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"verbose\", \"debug\", \"info\", \"warning\", \"error\", or \"silent\".",
-				), nil
+				)
 			}
 			if buildOpts != nil {
 				buildOpts.LogLevel = logLevel
@@ -589,11 +687,11 @@ func parseOptionsImpl(
 			}
 
 		case strings.HasPrefix(arg, "'--"):
-			return cli_helpers.MakeErrorWithNote(
+			return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 				fmt.Sprintf("Unexpected single quote character before flag: %s", arg),
 				"This typically happens when attempting to use single quotes to quote arguments with a shell that doesn't recognize single quotes. "+
 					"Try using double quote characters to quote arguments instead.",
-			), nil
+			)
 
 		case !strings.HasPrefix(arg, "-") && buildOpts != nil:
 			if equals := strings.IndexByte(arg, '='); equals != -1 {
@@ -611,7 +709,6 @@ func parseOptionsImpl(
 				"bundle":             true,
 				"ignore-annotations": true,
 				"keep-names":         true,
-				"metafile":           true,
 				"minify-identifiers": true,
 				"minify-syntax":      true,
 				"minify-whitespace":  true,
@@ -623,8 +720,10 @@ func parseOptionsImpl(
 			}
 
 			equals := map[string]bool{
+				"allow-overwrite":    true,
 				"asset-names":        true,
 				"banner":             true,
+				"bundle":             true,
 				"charset":            true,
 				"chunk-names":        true,
 				"color":              true,
@@ -633,28 +732,42 @@ func parseOptionsImpl(
 				"footer":             true,
 				"format":             true,
 				"global-name":        true,
+				"ignore-annotations": true,
 				"jsx-factory":        true,
 				"jsx-fragment":       true,
 				"jsx":                true,
+				"keep-names":         true,
 				"legal-comments":     true,
 				"loader":             true,
 				"log-level":          true,
 				"log-limit":          true,
 				"main-fields":        true,
+				"mangle-cache":       true,
+				"mangle-props":       true,
+				"mangle-quoted":      true,
+				"metafile":           true,
+				"minify-identifiers": true,
+				"minify-syntax":      true,
+				"minify-whitespace":  true,
+				"minify":             true,
 				"outbase":            true,
 				"outdir":             true,
 				"outfile":            true,
 				"platform":           true,
+				"preserve-symlinks":  true,
 				"public-path":        true,
+				"reserve-props":      true,
 				"resolve-extensions": true,
 				"source-root":        true,
 				"sourcefile":         true,
 				"sourcemap":          true,
 				"sources-content":    true,
+				"splitting":          true,
 				"target":             true,
 				"tree-shaking":       true,
 				"tsconfig-raw":       true,
 				"tsconfig":           true,
+				"watch":              true,
 			}
 
 			colon := map[string]bool{
@@ -713,9 +826,9 @@ func parseOptionsImpl(
 			}
 
 			if buildOpts != nil {
-				return cli_helpers.MakeErrorWithNote(fmt.Sprintf("Invalid build flag: %q", arg), note), nil
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(fmt.Sprintf("Invalid build flag: %q", arg), note)
 			} else {
-				return cli_helpers.MakeErrorWithNote(fmt.Sprintf("Invalid transform flag: %q", arg), note), nil
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(fmt.Sprintf("Invalid transform flag: %q", arg), note)
 			}
 		}
 	}
@@ -742,15 +855,18 @@ func parseTargets(targets []string, arg string) (target api.Target, engines []ap
 		"es2019": api.ES2019,
 		"es2020": api.ES2020,
 		"es2021": api.ES2021,
+		"es2022": api.ES2022,
 	}
 
 	validEngines := map[string]api.EngineName{
 		"chrome":  api.EngineChrome,
-		"firefox": api.EngineFirefox,
-		"safari":  api.EngineSafari,
 		"edge":    api.EngineEdge,
-		"node":    api.EngineNode,
+		"firefox": api.EngineFirefox,
+		"ie":      api.EngineIE,
 		"ios":     api.EngineIOS,
+		"node":    api.EngineNode,
+		"opera":   api.EngineOpera,
+		"safari":  api.EngineSafari,
 	}
 
 outer:
@@ -790,7 +906,7 @@ outer:
 }
 
 // This returns either BuildOptions, TransformOptions, or an error
-func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *string, *api.TransformOptions, *cli_helpers.ErrorWithNote) {
+func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *api.TransformOptions, parseOptionsExtras, *cli_helpers.ErrorWithNote) {
 	// If there's an entry point or we're bundling, then we're building
 	for _, arg := range osArgs {
 		if !strings.HasPrefix(arg, "-") || arg == "--bundle" {
@@ -801,11 +917,11 @@ func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *string, *api.Trans
 			options.LogLevel = api.LogLevelInfo
 			options.Write = true
 
-			err, metafile := parseOptionsImpl(osArgs, &options, nil, kindInternal)
+			extras, err := parseOptionsImpl(osArgs, &options, nil, kindInternal)
 			if err != nil {
-				return nil, nil, nil, err
+				return nil, nil, parseOptionsExtras{}, err
 			}
-			return &options, metafile, nil, nil
+			return &options, nil, extras, nil
 		}
 	}
 
@@ -816,9 +932,9 @@ func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *string, *api.Trans
 	options.LogLimit = 6
 	options.LogLevel = api.LogLevelInfo
 
-	err, _ := parseOptionsImpl(osArgs, nil, &options, kindInternal)
+	_, err := parseOptionsImpl(osArgs, nil, &options, kindInternal)
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, nil, parseOptionsExtras{}, err
 	}
 	if options.Sourcemap != api.SourceMapNone && options.Sourcemap != api.SourceMapInline {
 		var sourceMapMode string
@@ -830,13 +946,13 @@ func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *string, *api.Trans
 		case api.SourceMapLinked:
 			sourceMapMode = "linked"
 		}
-		return nil, nil, nil, cli_helpers.MakeErrorWithNote(
+		return nil, nil, parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 			fmt.Sprintf("Use \"--sourcemap\" instead of \"--sourcemap=%s\" when transforming stdin", sourceMapMode),
-			fmt.Sprintf("Using esbuild to transform stdin only generates one output file, so you cannot use the %q source map mode "+
+			fmt.Sprintf("Using esbuild to transform stdin only generates one output file. You cannot use the %q source map mode "+
 				"since that needs to generate two output files.", sourceMapMode),
 		)
 	}
-	return nil, nil, &options, nil
+	return nil, &options, parseOptionsExtras{}, nil
 }
 
 func splitWithEmptyCheck(s string, sep string) []string {
@@ -880,7 +996,7 @@ func runImpl(osArgs []string) int {
 	}
 	osArgs = osArgs[:end]
 
-	buildOptions, metafile, transformOptions, err := parseOptionsForRun(osArgs)
+	buildOptions, transformOptions, extras, err := parseOptionsForRun(osArgs)
 
 	switch {
 	case buildOptions != nil:
@@ -928,7 +1044,7 @@ func runImpl(osArgs []string) int {
 		// don't write any output files if it's incorrect. That makes this API
 		// option consistent with how we handle all other API options.
 		var writeMetafile func(string)
-		if metafile != nil {
+		if extras.metafile != nil {
 			var metafileAbsPath string
 			var metafileAbsDir string
 
@@ -939,9 +1055,9 @@ func runImpl(osArgs []string) int {
 			}
 			realFS, realFSErr := fs.RealFS(fs.RealFSOptions{AbsWorkingDir: buildOptions.AbsWorkingDir})
 			if realFSErr == nil {
-				absPath, ok := realFS.Abs(*metafile)
+				absPath, ok := realFS.Abs(*extras.metafile)
 				if !ok {
-					logger.PrintErrorToStderr(osArgs, fmt.Sprintf("Invalid metafile path: %s", *metafile))
+					logger.PrintErrorToStderr(osArgs, fmt.Sprintf("Invalid metafile path: %s", *extras.metafile))
 					return 1
 				}
 				metafileAbsPath = absPath
@@ -970,6 +1086,53 @@ func runImpl(osArgs []string) int {
 					}
 				}
 			}
+		}
+
+		// Also validate the mangle cache absolute path and directory ahead of time
+		// for the same reason
+		var writeMangleCache func(map[string]interface{})
+		if extras.mangleCache != nil {
+			var mangleCacheAbsPath string
+			var mangleCacheAbsDir string
+			var mangleCacheOrder []string
+			realFS, realFSErr := fs.RealFS(fs.RealFSOptions{AbsWorkingDir: buildOptions.AbsWorkingDir})
+			if realFSErr == nil {
+				absPath, ok := realFS.Abs(*extras.mangleCache)
+				if !ok {
+					logger.PrintErrorToStderr(osArgs, fmt.Sprintf("Invalid mangle cache path: %s", *extras.mangleCache))
+					return 1
+				}
+				mangleCacheAbsPath = absPath
+				mangleCacheAbsDir = realFS.Dir(absPath)
+				buildOptions.MangleCache, mangleCacheOrder = parseMangleCache(osArgs, realFS, *extras.mangleCache)
+				if buildOptions.MangleCache == nil {
+					return 1 // Stop now if parsing failed
+				}
+			} else {
+				// Don't fail in this case since the error will be reported by "api.Build"
+			}
+
+			writeMangleCache = func(mangleCache map[string]interface{}) {
+				if mangleCache == nil || realFSErr != nil {
+					return // Don't write out the metafile on build errors
+				}
+				if err != nil {
+					// This should already have been checked above
+					panic(err.Text)
+				}
+				fs.BeforeFileOpen()
+				defer fs.AfterFileClose()
+				if err := fs.MkdirAll(realFS, mangleCacheAbsDir, 0755); err != nil {
+					logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
+						"Failed to create output directory: %s", err.Error()))
+				} else {
+					bytes := printMangleCache(mangleCache, mangleCacheOrder, buildOptions.Charset == api.CharsetASCII)
+					if err := ioutil.WriteFile(mangleCacheAbsPath, bytes, 0644); err != nil {
+						logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
+							"Failed to write to output file: %s", err.Error()))
+					}
+				}
+			}
 
 			// Write out the metafile whenever we rebuild
 			if buildOptions.Watch != nil {
@@ -1003,6 +1166,11 @@ func runImpl(osArgs []string) int {
 			writeMetafile(result.Metafile)
 		}
 
+		// Write the mangle cache to the file system
+		if writeMangleCache != nil {
+			writeMangleCache(result.MangleCache)
+		}
+
 		// Do not exit if we're in watch mode
 		if buildOptions.Watch != nil {
 			<-make(chan bool)
@@ -1102,7 +1270,7 @@ func serveImpl(osArgs []string) error {
 	options.LogLimit = 5
 	options.LogLevel = api.LogLevelInfo
 
-	if err, _ := parseOptionsImpl(filteredArgs, &options, nil, kindInternal); err != nil {
+	if _, err := parseOptionsImpl(filteredArgs, &options, nil, kindInternal); err != nil {
 		logger.PrintErrorToStderr(filteredArgs, err.Text)
 		return errors.New(err.Text)
 	}
